#!/usr/bin/env python

# Copyright (C) 2006-2007 CEREA
#     Author: Vivien Mallet
#
# CEREA (http://www.enpc.fr/cerea) is a joint laboratory of
# ENPC (http://www.enpc.fr/) and EDF R&D (http://www.edf.fr/).
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License (file "license") for more details.

import sys, datetime
from numpy import *
from atmopy import *

if len(sys.argv) == 1: # no argument is provided.
    input_file = "../results/O3.bin"
else:
    input_file = sys.argv[1]

# 1st July 2001 at 01:00 UT.
t_min = datetime.datetime(2001, 07, 01, 01)
# Longitude and latitude.
x_min = -10.
y_min = 36.

# Timestep: one hour.
Delta_t = 1.
# Two degrees (horizontal resolution).
Delta_x = 2.
Delta_y = 2.

# Number of hours.
Nt = 48

Nx = 16
Ny = 11
# Concentrations are saved at ground level only.
Nz = 1

origin = (t_min, y_min, x_min)
delta = (Delta_t, Delta_y, Delta_x)
shape = (Nt, Ny, Nx)

# Generates the list of simulated dates.
date = observation.get_simulation_dates(t_min, Delta_t, Nt)

# Monitoring stations.
obs_dir = "../raw_data/obs/"
station_file = obs_dir + "stations"
station_file_type = "Emep"
# Loads data (name, type, position) of all stations in the domain.
station_all = io.load_stations(station_file, station_file_type,
                               origin[1:], delta[1:], shape[1:])
# List of stations in the domain and with observations available.
station_list = []

################
# OBSERVATIONS #
################

obs_date = []
obs = []
for station in station_all:
    # Loads observations and associated dates.
    tmp_date, value = io.load_file_observations(station.name, obs_dir)
    tmp_date, value = observation.remove_missing(tmp_date, value, (-999, 0))
    # Removes concentrations outside the considered period.
    tmp_date, value = observation.restrict_to_period(tmp_date, value,
                                                     [date[0], date[-1]])

    # Only stations with available observations are kept.
    if len(value) != 0:
        station_list.append(station)
        obs_date.append(tmp_date)
        obs.append(array(value, dtype = 'd'))

##############
# SIMULATION #
##############

# Reads the computed concentrations and extracts the first level.
sim3d = io.load_binary(input_file, [Nt, Nz, Ny, Nx])[:, 0]

# Simulated concentrations at stations (with observations).
sim_date = []
sim = []
for istation in range(len(station_list)):
    station = station_list[istation]
    # Extracts simulated data.
    tmp = observation.get_simulated_at_station_closest(origin, delta,
                                                       sim3d, station)
    # Keeps data (simulated and observed) when there are observations AND
    # simulated concentrations. Here, there is a simulated concentration for
    # every observation: only simulated concentrations are filtered out.
    tmp_date, tmp, o = observation.restrict_to_common_days(date, tmp,
                                                           obs_date[istation],
                                                           obs[istation])
    sim_date.append(tmp_date)
    sim.append(tmp)

# Computes statistics.
rmse = []
for d in date:
    # Retrieves simulated and observed data for date 'd', at all stations.
    s, o = ensemble.collect(sim, obs, dates = obs_date, period = d)
    tmp = (s - o) * (s - o)
    rmse.append(sqrt(tmp.sum() / float(tmp.size)))

print "  ".join(["%.1f" % x for x in rmse])
rmse = array(rmse)
print "Mean: %.2f" % rmse.mean()
